# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.message import Message as _Message

__all__ = [
    "PruneRequest",
    "PruneResponse",
]


class PruneRequest(_Message):
    prune_up_to: _builtins.str
    submission_id: _builtins.str
    prune_all_divulged_contracts: _builtins.bool
    def __init__(self, *, prune_up_to: _typing.Optional[_builtins.str] = ..., submission_id: _typing.Optional[_builtins.str] = ..., prune_all_divulged_contracts: _typing.Optional[_builtins.bool] = ...): ...
    def HasField(self, field_name: _typing.Literal["prune_up_to", "submission_id", "prune_all_divulged_contracts"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["prune_up_to", "submission_id", "prune_all_divulged_contracts"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class PruneResponse(_Message):
    def __init__(self): ...
    def HasField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def ClearField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from google.protobuf.compiler.plugin_pb2 import CodeGeneratorRequest, CodeGeneratorResponse

from .. import protoc

__all__ = ["run_plugin"]


def run_plugin(request: CodeGeneratorRequest) -> CodeGeneratorResponse:
    # the default Go plugin already seems to respect copyright headers, so we don't actually
    # need to make any changes to the generated code
    return protoc.run_plugin("go", request)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, typing as _typing

import grpc as _grpc
from grpc import aio as _grpc_aio

from .identity_provider_config_service_pb2 import CreateIdentityProviderConfigRequest, CreateIdentityProviderConfigResponse, DeleteIdentityProviderConfigRequest, DeleteIdentityProviderConfigResponse, GetIdentityProviderConfigRequest, GetIdentityProviderConfigResponse, ListIdentityProviderConfigsRequest, ListIdentityProviderConfigsResponse, UpdateIdentityProviderConfigRequest, UpdateIdentityProviderConfigResponse

__all__ = [
    "IdentityProviderConfigServiceStub",
]


# noinspection PyPep8Naming,DuplicatedCode
class IdentityProviderConfigServiceStub:
    @classmethod  # type: ignore
    @_typing.overload
    def __new__(cls, channel: _grpc.Channel) -> _IdentityProviderConfigServiceBlockingStub: ...  # type: ignore
    @classmethod  # type: ignore
    @_typing.overload
    def __new__(cls, channel: _grpc_aio.Channel) -> _IdentityProviderConfigServiceAsyncStub: ...  # type: ignore
    def CreateIdentityProviderConfig(self, __1: CreateIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[CreateIdentityProviderConfigResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, CreateIdentityProviderConfigResponse]]: ...
    def GetIdentityProviderConfig(self, __1: GetIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[GetIdentityProviderConfigResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, GetIdentityProviderConfigResponse]]: ...
    def UpdateIdentityProviderConfig(self, __1: UpdateIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[UpdateIdentityProviderConfigResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, UpdateIdentityProviderConfigResponse]]: ...
    def ListIdentityProviderConfigs(self, __1: ListIdentityProviderConfigsRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListIdentityProviderConfigsResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, ListIdentityProviderConfigsResponse]]: ...
    def DeleteIdentityProviderConfig(self, __1: DeleteIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[DeleteIdentityProviderConfigResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, DeleteIdentityProviderConfigResponse]]: ...

# noinspection PyPep8Naming,DuplicatedCode
class _IdentityProviderConfigServiceBlockingStub(IdentityProviderConfigServiceStub):
    def CreateIdentityProviderConfig(self, __1: CreateIdentityProviderConfigRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> CreateIdentityProviderConfigResponse: ...
    def GetIdentityProviderConfig(self, __1: GetIdentityProviderConfigRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> GetIdentityProviderConfigResponse: ...
    def UpdateIdentityProviderConfig(self, __1: UpdateIdentityProviderConfigRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> UpdateIdentityProviderConfigResponse: ...
    def ListIdentityProviderConfigs(self, __1: ListIdentityProviderConfigsRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListIdentityProviderConfigsResponse: ...
    def DeleteIdentityProviderConfig(self, __1: DeleteIdentityProviderConfigRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> DeleteIdentityProviderConfigResponse: ...

# noinspection PyPep8Naming,DuplicatedCode
class _IdentityProviderConfigServiceAsyncStub(IdentityProviderConfigServiceStub):
    def CreateIdentityProviderConfig(self, __1: CreateIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, CreateIdentityProviderConfigResponse]: ...  # type: ignore
    def GetIdentityProviderConfig(self, __1: GetIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, GetIdentityProviderConfigResponse]: ...  # type: ignore
    def UpdateIdentityProviderConfig(self, __1: UpdateIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, UpdateIdentityProviderConfigResponse]: ...  # type: ignore
    def ListIdentityProviderConfigs(self, __1: ListIdentityProviderConfigsRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListIdentityProviderConfigsResponse]: ...  # type: ignore
    def DeleteIdentityProviderConfig(self, __1: DeleteIdentityProviderConfigRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, DeleteIdentityProviderConfigResponse]: ...  # type: ignore



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.any_pb2 import Any
from google.protobuf.internal.containers import RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
from google.protobuf.message import Message as _Message
from google.protobuf.wrappers_pb2 import StringValue
from google.rpc.status_pb2 import Status

from .contract_metadata_pb2 import ContractMetadata
from .value_pb2 import Identifier, Record, Value

__all__ = [
    "Event",
    "CreatedEvent",
    "InterfaceView",
    "ArchivedEvent",
    "ExercisedEvent",
]


class Event(_Message):
    @property
    def created(self) -> CreatedEvent: ...
    @property
    def archived(self) -> ArchivedEvent: ...
    @_typing.overload
    def __init__(self): ...
    @_typing.overload
    def __init__(self, *, created: CreatedEvent = ...): ...
    @_typing.overload
    def __init__(self, *, archived: ArchivedEvent = ...): ...
    def HasField(self, field_name: _typing.Literal["event", "created", "archived"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["event", "created", "archived"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.Literal["event"]) -> _typing.Literal[None, "created", "archived"]: ...

class CreatedEvent(_Message):
    event_id: _builtins.str
    contract_id: _builtins.str
    @property
    def template_id(self) -> Identifier: ...
    @property
    def contract_key(self) -> Value: ...
    @property
    def create_arguments(self) -> Record: ...
    @property
    def create_arguments_blob(self) -> Any: ...
    @property
    def interface_views(self) -> RepeatedCompositeFieldContainer[InterfaceView]: ...
    @property
    def witness_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    @property
    def signatories(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    @property
    def observers(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    @property
    def agreement_text(self) -> StringValue: ...
    @property
    def metadata(self) -> ContractMetadata: ...
    def __init__(self, *, event_id: _typing.Optional[_builtins.str] = ..., contract_id: _typing.Optional[_builtins.str] = ..., template_id: _typing.Optional[Identifier] = ..., contract_key: _typing.Optional[Value] = ..., create_arguments: _typing.Optional[Record] = ..., create_arguments_blob: _typing.Optional[Any] = ..., interface_views: _typing.Optional[_typing.Iterable[InterfaceView]] = ..., witness_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., signatories: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., observers: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., agreement_text: _typing.Optional[StringValue] = ..., metadata: _typing.Optional[ContractMetadata] = ...): ...
    def HasField(self, field_name: _typing.Literal["event_id", "contract_id", "template_id", "contract_key", "create_arguments", "create_arguments_blob", "interface_views", "witness_parties", "signatories", "observers", "agreement_text", "metadata"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["event_id", "contract_id", "template_id", "contract_key", "create_arguments", "create_arguments_blob", "interface_views", "witness_parties", "signatories", "observers", "agreement_text", "metadata"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class InterfaceView(_Message):
    @property
    def interface_id(self) -> Identifier: ...
    @property
    def view_status(self) -> Status: ...
    @property
    def view_value(self) -> Record: ...
    def __init__(self, *, interface_id: _typing.Optional[Identifier] = ..., view_status: _typing.Optional[Status] = ..., view_value: _typing.Optional[Record] = ...): ...
    def HasField(self, field_name: _typing.Literal["interface_id", "view_status", "view_value"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["interface_id", "view_status", "view_value"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ArchivedEvent(_Message):
    event_id: _builtins.str
    contract_id: _builtins.str
    @property
    def template_id(self) -> Identifier: ...
    @property
    def witness_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(self, *, event_id: _typing.Optional[_builtins.str] = ..., contract_id: _typing.Optional[_builtins.str] = ..., template_id: _typing.Optional[Identifier] = ..., witness_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ...): ...
    def HasField(self, field_name: _typing.Literal["event_id", "contract_id", "template_id", "witness_parties"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["event_id", "contract_id", "template_id", "witness_parties"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ExercisedEvent(_Message):
    event_id: _builtins.str
    contract_id: _builtins.str
    @property
    def template_id(self) -> Identifier: ...
    @property
    def interface_id(self) -> Identifier: ...
    choice: _builtins.str
    @property
    def choice_argument(self) -> Value: ...
    @property
    def acting_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    consuming: _builtins.bool
    @property
    def witness_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    @property
    def child_event_ids(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    @property
    def exercise_result(self) -> Value: ...
    def __init__(self, *, event_id: _typing.Optional[_builtins.str] = ..., contract_id: _typing.Optional[_builtins.str] = ..., template_id: _typing.Optional[Identifier] = ..., interface_id: _typing.Optional[Identifier] = ..., choice: _typing.Optional[_builtins.str] = ..., choice_argument: _typing.Optional[Value] = ..., acting_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., consuming: _typing.Optional[_builtins.bool] = ..., witness_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., child_event_ids: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., exercise_result: _typing.Optional[Value] = ...): ...
    def HasField(self, field_name: _typing.Literal["event_id", "contract_id", "template_id", "interface_id", "choice", "choice_argument", "acting_parties", "consuming", "witness_parties", "child_event_ids", "exercise_result"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["event_id", "contract_id", "template_id", "interface_id", "choice", "choice_argument", "acting_parties", "consuming", "witness_parties", "child_event_ids", "exercise_result"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
DAML-LF fast lookups
--------------------
"""

from __future__ import annotations

# Implementation notes:
#
# The code in here is fairly monotonous and boilerplate heavy. However, being too creative here can
# potentially lead to performance degradations, particularly at application startup where type and
# template lookups by name are very frequent. Please be conscious of the runtime costs of
# modifications in this file!
import threading
from types import MappingProxyType
from typing import (
    AbstractSet,
    Any,
    Collection,
    Dict,
    Iterable,
    List,
    NoReturn,
    Optional,
    Sequence,
    Set,
    Tuple,
    Union,
)
import warnings

from .. import LOG
from .daml_lf_1 import (
    Archive,
    DefDataType,
    DefTemplate,
    DefValue,
    DottedName,
    ModuleRef,
    Package,
    PackageRef,
    TemplateChoice,
    TypeConName,
    ValName,
)
from .errors import NameNotFoundError, PackageNotFoundError
from .protocols import SymbolLookup
from .util import package_local_name, package_ref

__all__ = [
    "STAR",
    "EmptyLookup",
    "MultiPackageLookup",
    "PackageLookup",
    "PackageExceptionTracker",
    "SymbolLookup",
    "find_choice",
    "matching_normalizations",
    "normalize",
    "parse_type_con_name",
    "validate_template",
]

STAR = PackageRef("*")


def parse_type_con_name(val: str, /) -> TypeConName:
    """
    Parse the given string as a type constructor.
    """
    pkg, name = validate_template(val)
    module_name, _, entity_name = name.rpartition(":")
    if module_name:
        module_ref = ModuleRef(pkg, DottedName(module_name.split(".")))
    else:
        module_ref = ModuleRef(pkg, DottedName())
    return TypeConName(module_ref, entity_name.split("."))


def empty_lookup_impl(ref: Any, /) -> NoReturn:
    pkg, _ = validate_template(ref)
    if pkg != STAR:
        raise PackageNotFoundError(pkg)
    else:
        raise NameNotFoundError(ref)


def validate_template(template: Any, /) -> Tuple[PackageRef, str]:
    """
    Return a module and type name component from something that can be interpreted as a template.

    :param template:
        Any object that can be interpreted as an identifier for a template.
    :return:
        A tuple of package ID and ``Module.Name:EntityName`` (the package-scoped identifier for the
        type). The special value ``'*'`` is used if either the package ID, module name, or both
        should be wildcarded.
    :raise ValueError:
        If the object could not be interpreted as a thing referring to a template.
    """
    if template == "*" or template is None:
        return STAR, "*"

    if isinstance(template, str):
        components = template.split(":")
        if len(components) == 3:
            # correct number of colons for a fully-qualified name
            pkgid, m, e = components
            return PackageRef(pkgid), f"{m}:{e}"

        elif len(components) == 2:
            # one colon, so assume the package ID is unspecified UNLESS the second component is a
            # wildcard; then we assume the wildcard means any module name and entity name
            m, e = components
            if m == STAR and e != STAR:
                # strings that look like "*:SOMETHING" are explicitly not allowed unless deprecated
                # identifier support is requested; this is almost certainly an attempt to use
                # periods instead of colons as a delimiter between module name and entity name
                raise ValueError("string must be in the format PKG_REF:MOD:ENTITY or MOD:ENTITY")

            return (STAR, f"{m}:{e}") if e != "*" else (PackageRef(m), "*")

        else:
            raise ValueError("string must be in the format PKG_REF:MOD:ENTITY or MOD:ENTITY")

    if isinstance(template, TypeConName):
        return package_ref(template), package_local_name(template)
    else:
        raise ValueError(f"Don't know how to convert {template!r} into a template")


def normalize(name: Union[None, str, TypeConName], /) -> str:
    """
    Return the canonical form for a string that represents a template ID or partial match of a
    template ID.

    Concretely, this function converts ``"MyMod:MyTemplate"`` to ``"*:MyMod:MyTemplate"`` and leaves
    most other strings unchanged.

    :param name:
        A template ID, expressed in either string form or as an instance of :class:`TypeConName`.
    :return:
        A string in canonical form (either ``PACKAGE_REF:MODULE_NAME:ENTITY_NAME`` or
        ``PACKAGE_REF:*``, where ``PACKAGE_REF`` is also allowed to be ``*``).
    """
    p, m = validate_template(name)
    return f"{p}:{m}"


def matching_normalizations(name: Union[str, TypeConName], /) -> Sequence[str]:
    """
    Return strings that are possible matches for the given template ID.
    """
    p, m = validate_template(name)

    # throw away duplicates that arise from `name` not being fully specified (p and/or m are
    # allowed to be asterisks too)
    return list(dict.fromkeys([f"{p}:{m}", f"{p}:*", f"*:{m}", "*:*"]))


class EmptyLookup(SymbolLookup):
    """
    A :class:`SymbolLookup` that trivially throws for all of its functions.

    This can be used where a :class:`SymbolLookup` instance is useful but an implementation is not
    required.

    All methods are implemented such that if the provided ref has a package ID,
    :class:`PackageNotFoundError` is thrown; otherwise, :class:`NameNotFoundError` is thrown.
    """

    __slots__ = ()

    def archives(self) -> Collection[Archive]:
        return frozenset()

    def package_ids(self) -> AbstractSet[PackageRef]:
        return frozenset()

    def data_type_name(self, ref: Any) -> NoReturn:
        raise empty_lookup_impl(ref)

    def data_type(self, ref: Any) -> NoReturn:
        raise empty_lookup_impl(ref)

    def value(self, ref: Any) -> NoReturn:
        raise empty_lookup_impl(ref)

    def template_names(self, ref: Any) -> Collection[TypeConName]:
        return frozenset()

    def template_name(self, ref: Any) -> NoReturn:
        raise empty_lookup_impl(ref)

    def template(self, ref: Any) -> NoReturn:
        raise empty_lookup_impl(ref)


class PackageLookup(SymbolLookup):
    """
    Caching structure to make lookups on type names within a :class:`Package` faster.
    """

    def __init__(self, archive: Archive):
        self.archive = archive

        data_types = {}  # type: Dict[str, Tuple[TypeConName, DefDataType]]
        values = {}  # type: Dict[str, Tuple[ValName, DefValue]]
        templates = {}  # type: Dict[str, Tuple[TypeConName, DefTemplate]]
        for module in self.archive.package.modules:
            module_ref = ModuleRef(archive.hash, module.name)

            for dt in module.data_types:
                dt_name = TypeConName(module_ref, dt.name.segments)
                data_types[f"{module.name}:{dt.name}"] = (dt_name, dt)

            for value in module.values:
                value_name = ValName(module_ref, value.name_with_type.name)
                values[f"{module.name}:{value.name_with_type.name}"] = (value_name, value)

            for tmpl in module.templates:
                tmpl_name = TypeConName(module_ref, tmpl.tycon.segments)
                templates[f"{module.name}:{tmpl.tycon}"] = (tmpl_name, tmpl)

        self._data_types = MappingProxyType(data_types)
        self._values = MappingProxyType(values)
        self._templates = MappingProxyType(templates)

    def archives(self) -> Collection[Archive]:
        return [self.archive]

    def package_ids(self) -> AbstractSet[PackageRef]:
        return frozenset([self.archive.hash])

    def data_type_name(self, ref: Any) -> TypeConName:
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            dt_name = self.local_data_type_name(name)
            if dt_name is not None:
                return dt_name

        raise NameNotFoundError(ref)

    def data_type(self, ref: Any) -> DefDataType:
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            dt = self.local_data_type(name)
            if dt is not None:
                return dt

        raise NameNotFoundError(ref)

    def local_data_type_name(self, name: str) -> Optional[TypeConName]:
        r = self._data_types.get(name)
        return r[0] if r is not None else None

    def local_data_type(self, name: str) -> Optional[DefDataType]:
        """
        Variation of :meth:`data_type` that assumes the name is already scoped to this package.
        Unlike :meth:`data_type`, this method returns ``None`` in the case of no match.

        You should not normally use this method directly, and instead prefer to use the methods of
        the :class:`SymbolLookup` protocol.

        :param name:
            A name to search for. Must be of the form ``"ModuleName:EntityName"``, where both
            modules and entities are dot-delimited.
        :return:
            Either a matching :class:`DefDataType`, or ``None`` if no match.
        """
        r = self._data_types.get(name)
        return r[1] if r is not None else None

    def value(self, ref: Any) -> DefValue:
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            dt = self.local_value(name)
            if dt is not None:
                return dt

        raise NameNotFoundError(ref)

    def local_value(self, name: str) -> Optional[DefValue]:
        """
        Variation of :meth:`data_type` that assumes the name is already scoped to this package.
        Unlike :meth:`data_type`, this method returns ``None`` in the case of no match.

        You should not normally use this method directly, and instead prefer to use the methods of
        the :class:`SymbolLookup` protocol.

        :param name:
            A name to search for. Must be of the form ``"ModuleName:EntityName"``, where both
            modules and entities are dot-delimited.
        :return:
            Either a matching :class:`DefDataType`, or ``None`` if no match.
        """
        r = self._values.get(name)
        return r[1] if r is not None else None

    def template_names(self, ref: Any) -> Collection[TypeConName]:
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            if name == "*":
                return self.local_template_names()
            elif name in self._templates:
                n, _ = self._templates[name]
                return [n]
        return []

    def template_name(self, ref: Any) -> TypeConName:
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            tmpl = self.local_template_name(name)
            if tmpl is not None:
                return tmpl

        raise NameNotFoundError(ref)

    def template(self, ref: Any) -> DefTemplate:
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            tmpl = self.local_template(name)
            if tmpl is not None:
                return tmpl

        raise NameNotFoundError(ref)

    def local_template_names(self) -> Collection[TypeConName]:
        return [n for n, _ in self._templates.values()]

    def local_template_name(self, name: str) -> Optional[TypeConName]:
        r = self._templates.get(name)
        return r[0] if r is not None else None

    def local_template(self, name: str) -> Optional[DefTemplate]:
        """
        Variation of :meth:`data_type` that assumes the name is already scoped to this package.
        Unlike :meth:`data_type`, this method returns ``None`` in the case of no match.

        You should not normally use this method directly, and instead prefer to use the methods of
        the :class:`SymbolLookup` protocol.

        :param name:
            A name to search for. Must be of the form ``"ModuleName:EntityName"``, where both
            modules and entities are dot-delimited.
        :return:
            Either a matching :class:`DefDataType`, or ``None`` if no match.
        """
        r = self._templates.get(name)
        return r[1] if r is not None else None


class MultiPackageLookup(SymbolLookup):
    """
    Combines lookups across multiple archives.

    This class is thread-safe. When calling :meth:`add_archive` and any of the other read-only
    methods concurrently, read-only methods will NOT block; they will return the previous state
    of the lookup.

    Packages can only be added; they cannot be removed once added.
    """

    def __init__(self, archives: Optional[Collection[Archive]] = None):
        self._lock = threading.Lock()
        self._cache = {}  # type: Dict[PackageRef, PackageLookup]
        if archives is not None:
            self.add_archive(*archives)

    def archives(self) -> Collection[Archive]:
        """
        Return the list of known archives.
        """
        return [lookup.archive for lookup in self._cache.values()]

    def add_archive(self, *a: Archive) -> None:
        """
        Add one or more :class:`Archive` instances to this lookup.

        This method is thread-safe, but note that :class:`MultiPackageLookup` allows dirty reads for
        performance reasons.

        :param a: One or more :class:`Archive` instances to add.
        """
        new_lookups = {ar.hash: PackageLookup(ar) for ar in a}
        with self._lock:
            # replace the old cache with a new one, incorporating values from both the existing
            # cache and the new lookups that came in. When there is a key conflict between the two
            # APIs, always prefer the existing cache to provide stability to callers.
            self._cache = {**new_lookups, **self._cache}
            LOG.debug("Updated package cache; now contains %d packages.", len(self._cache))

    def package_ids(self) -> AbstractSet[PackageRef]:
        """
        Return the package IDs of packages stored in this lookup.

        Because the data is local and in memory, the timeout parameter is ignored.
        """
        return set(self._cache)

    def package(self, ref: PackageRef) -> Package:
        lookup = self._cache.get(ref)
        if lookup is not None:
            return lookup.archive.package

        raise PackageNotFoundError(ref)

    def data_type_name(self, ref: Any) -> TypeConName:
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            dt_name = lookup.local_data_type_name(name)
            if dt_name is not None:
                return dt_name

        raise NameNotFoundError(ref)

    def data_type(self, ref: Any) -> DefDataType:
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            dt = lookup.local_data_type(name)
            if dt is not None:
                return dt

        raise NameNotFoundError(ref)

    def value(self, ref: Any) -> DefValue:
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            val = lookup.local_value(name)
            if val is not None:
                return val

        raise NameNotFoundError(ref)

    def template_names(self, ref: Any) -> Collection[TypeConName]:
        names = []  # type: List[TypeConName]

        pkg, name = validate_template(ref)
        try:
            lookups = self._lookups(pkg)
        except PackageNotFoundError:
            return []

        for lookup in lookups:
            if name == "*":
                names.extend(lookup.local_template_names())
            else:
                n = lookup.local_template_name(name)
                if n is not None:
                    names.append(n)

        return names

    def template_name(self, ref: Any) -> TypeConName:
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            n = lookup.local_template_name(name)
            if n is not None:
                return n

        raise NameNotFoundError(ref)

    def template(self, ref: Any) -> DefTemplate:
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            tmpl = lookup.local_template(name)
            if tmpl is not None:
                return tmpl

        raise NameNotFoundError(ref)

    def _lookups(self, ref: PackageRef) -> Iterable[PackageLookup]:
        """
        Return the individual :class:`PackageLookup` objects that should be consulted based on the
        :class:`PackageRef`.

        :param ref:
            A :class:`PackageRef` to look for.
        :return:
            A collection of :class:`PackageLookup` objects that match the :class:`PackageRef`. This
            collection is never empty.
        :raises PackageNotFoundError:
            if the :class:`PackageRef` points to a package that is not present in this lookup.
        """
        if ref == "*":
            return self._cache.values()

        lookup = self._cache.get(ref)
        if lookup is not None:
            return (lookup,)

        raise PackageNotFoundError(ref)


class PackageExceptionTracker:
    """
    A context manager that gracefully recovers from errors related to missing packages that are
    retryable.
    """

    def __init__(self) -> None:
        self._seen_types = set()  # type: Set[str]
        self._seen_packages = set()  # type: Set[PackageRef]
        self._pkg_refs = list()  # type: List[PackageRef]
        self.done = False

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if isinstance(exc_val, PackageNotFoundError):
            # every time we fail serialization due to a missing package or type,
            # try to resolve it; remember what we tried, because if we fail again
            # for the same reason it is likely fatal
            if exc_val.ref in self._seen_packages:
                # we already looked for this package and couldn't find it; this will
                # never succeed, so we can't suppress this error
                return None

            self._seen_packages.add(exc_val.ref)

            # the caller should try to load the offending package, and then retry the operation
            self._pkg_refs.append(exc_val.ref)
            return True

        elif isinstance(exc_val, NameNotFoundError):
            if exc_val.ref in self._seen_types:
                # we already looked for this type and couldn't find it; this will
                # never succeed
                LOG.verbose(
                    "Failed to find name %s in all known packages, "
                    "even after fetching the latest.",
                    exc_val.ref,
                )
                raise

            with warnings.catch_warnings():
                warnings.simplefilter("ignore", DeprecationWarning)
                pkg_id, name = validate_template(exc_val.ref)
            if pkg_id == "*":
                # we don't know what package contains this type, so we have no
                # choice but to look in all known packages
                LOG.verbose(
                    "Failed to find name %s in all known packages, " "so loading ALL packages...",
                    name,
                )
                self._seen_types.add(exc_val.ref)

                # the caller should load all packages, and then retry the operation
                self._pkg_refs.append(STAR)
                return True
            else:
                # we know what package this type comes from, but it did not contain
                # the required type
                LOG.warning("Found package %s, but it did not include type %s", pkg_id, name)
                return None

    def pop_package(self) -> Optional[PackageRef]:
        """
        Return a :class:`PackageRef` that should be fetched before an operation is retried, or
        ``None`` if there is no such operation.
        """
        return self._pkg_refs.pop() if self._pkg_refs else None


def find_choice(template: DefTemplate, name: str) -> TemplateChoice:
    """
    Find a choice in a :class:`DefTemplate`. If the choice could not be found,
    :class:`NameNotFoundError` is raised.
    """
    for choice in template.choices:
        if choice.name == name:
            return choice

    raise NameNotFoundError(f"choice {name}")



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import party_management_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2


class PartyManagementServiceStub(object):
    """This service allows inspecting the party management state of the ledger known to the participant
    and managing the participant-local party metadata.

    The authorization rules for its RPCs are specified on the ``<RpcName>Request``
    messages as boolean expressions over these facts:
    (1) ``HasRight(r)`` denoting whether the authenticated user has right ``r`` and
    (2) ``IsAuthenticatedIdentityProviderAdmin(idp)`` denoting whether ``idp`` is equal to the ``identity_provider_id``
    of the authenticated user and the user has an IdentityProviderAdmin right.

    The fields of request messages (and sub-messages) are marked either as ``Optional`` or ``Required``:
    (1) ``Optional`` denoting the client may leave the field unset when sending a request.
    (2) ``Required`` denoting the client must set the field to a non-default value when sending a request.

    A party details resource is described by the ``PartyDetails`` message,
    A party details resource, once it has been created, can be modified using the ``UpdatePartyDetails`` RPC.
    The only fields that can be modified are those marked as ``Modifiable``.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetParticipantId = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.PartyManagementService/GetParticipantId',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetParticipantIdRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetParticipantIdResponse.FromString,
                )
        self.GetParties = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.PartyManagementService/GetParties',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetPartiesRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetPartiesResponse.FromString,
                )
        self.ListKnownParties = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.PartyManagementService/ListKnownParties',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.ListKnownPartiesRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.ListKnownPartiesResponse.FromString,
                )
        self.AllocateParty = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.PartyManagementService/AllocateParty',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.AllocatePartyRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.AllocatePartyResponse.FromString,
                )
        self.UpdatePartyDetails = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.PartyManagementService/UpdatePartyDetails',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.UpdatePartyDetailsRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.UpdatePartyDetailsResponse.FromString,
                )


class PartyManagementServiceServicer(object):
    """This service allows inspecting the party management state of the ledger known to the participant
    and managing the participant-local party metadata.

    The authorization rules for its RPCs are specified on the ``<RpcName>Request``
    messages as boolean expressions over these facts:
    (1) ``HasRight(r)`` denoting whether the authenticated user has right ``r`` and
    (2) ``IsAuthenticatedIdentityProviderAdmin(idp)`` denoting whether ``idp`` is equal to the ``identity_provider_id``
    of the authenticated user and the user has an IdentityProviderAdmin right.

    The fields of request messages (and sub-messages) are marked either as ``Optional`` or ``Required``:
    (1) ``Optional`` denoting the client may leave the field unset when sending a request.
    (2) ``Required`` denoting the client must set the field to a non-default value when sending a request.

    A party details resource is described by the ``PartyDetails`` message,
    A party details resource, once it has been created, can be modified using the ``UpdatePartyDetails`` RPC.
    The only fields that can be modified are those marked as ``Modifiable``.
    """

    def GetParticipantId(self, request, context):
        """Return the identifier of the participant.
        All horizontally scaled replicas should return the same id.
        daml-on-kv-ledger: returns an identifier supplied on command line at launch time
        canton: returns globally unique identifier of the participant
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetParties(self, request, context):
        """Get the party details of the given parties. Only known parties will be
        returned in the list.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListKnownParties(self, request, context):
        """List the parties known by the participant.
        The list returned contains parties whose ledger access is facilitated by
        the participant and the ones maintained elsewhere.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AllocateParty(self, request, context):
        """Allocates a new party on a ledger and adds it to the set managed by the participant.
        Caller specifies a party identifier suggestion, the actual identifier
        allocated might be different and is implementation specific.
        Caller can specify party metadata that is stored locally on the participant.
        This call may:
        - Succeed, in which case the actual allocated identifier is visible in
        the response.
        - Respond with a gRPC error
        daml-on-kv-ledger: suggestion's uniqueness is checked by the validators in
        the consensus layer and call rejected if the identifier is already present.
        canton: completely different globally unique identifier is allocated.
        Behind the scenes calls to an internal protocol are made. As that protocol
        is richer than the surface protocol, the arguments take implicit values
        The party identifier suggestion must be a valid party name. Party names are required to be non-empty US-ASCII strings built from letters, digits, space,
        colon, minus and underscore limited to 255 chars
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdatePartyDetails(self, request, context):
        """Update selected modifiable participant-local attributes of a party details resource.
        Can update the participant's local information for local parties.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PartyManagementServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetParticipantId': grpc.unary_unary_rpc_method_handler(
                    servicer.GetParticipantId,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetParticipantIdRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetParticipantIdResponse.SerializeToString,
            ),
            'GetParties': grpc.unary_unary_rpc_method_handler(
                    servicer.GetParties,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetPartiesRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetPartiesResponse.SerializeToString,
            ),
            'ListKnownParties': grpc.unary_unary_rpc_method_handler(
                    servicer.ListKnownParties,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.ListKnownPartiesRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.ListKnownPartiesResponse.SerializeToString,
            ),
            'AllocateParty': grpc.unary_unary_rpc_method_handler(
                    servicer.AllocateParty,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.AllocatePartyRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.AllocatePartyResponse.SerializeToString,
            ),
            'UpdatePartyDetails': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdatePartyDetails,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.UpdatePartyDetailsRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.UpdatePartyDetailsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.daml.ledger.api.v1.admin.PartyManagementService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PartyManagementService(object):
    """This service allows inspecting the party management state of the ledger known to the participant
    and managing the participant-local party metadata.

    The authorization rules for its RPCs are specified on the ``<RpcName>Request``
    messages as boolean expressions over these facts:
    (1) ``HasRight(r)`` denoting whether the authenticated user has right ``r`` and
    (2) ``IsAuthenticatedIdentityProviderAdmin(idp)`` denoting whether ``idp`` is equal to the ``identity_provider_id``
    of the authenticated user and the user has an IdentityProviderAdmin right.

    The fields of request messages (and sub-messages) are marked either as ``Optional`` or ``Required``:
    (1) ``Optional`` denoting the client may leave the field unset when sending a request.
    (2) ``Required`` denoting the client must set the field to a non-default value when sending a request.

    A party details resource is described by the ``PartyDetails`` message,
    A party details resource, once it has been created, can be modified using the ``UpdatePartyDetails`` RPC.
    The only fields that can be modified are those marked as ``Modifiable``.
    """

    @staticmethod
    def GetParticipantId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.PartyManagementService/GetParticipantId',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetParticipantIdRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetParticipantIdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetParties(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.PartyManagementService/GetParties',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetPartiesRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.GetPartiesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListKnownParties(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.PartyManagementService/ListKnownParties',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.ListKnownPartiesRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.ListKnownPartiesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AllocateParty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.PartyManagementService/AllocateParty',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.AllocatePartyRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.AllocatePartyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdatePartyDetails(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.PartyManagementService/UpdatePartyDetails',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.UpdatePartyDetailsRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_party__management__service__pb2.UpdatePartyDetailsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
"""
This module contains the types needed to submit commands to and read events from a
Daml `gRPC Ledger API <https://docs.daml.com/app-dev/ledger-api.html>`_ or
`HTTP JSON API <https://docs.daml.com/json-api/index.html>`_.
"""
from __future__ import annotations

from typing import Protocol, runtime_checkable

from .api_types import (
    ActAs,
    Admin,
    ArchiveEvent,
    Boundary,
    Command,
    CommandMeta,
    CreateAndExerciseCommand,
    CreateCommand,
    CreateEvent,
    Event,
    EventOrBoundary,
    ExerciseByKeyCommand,
    ExerciseCommand,
    ExerciseResponse,
    MeteringReport,
    PartyInfo,
    ReadAs,
    Right,
    SubmitResponse,
    User,
)

__all__ = [
    "aio",
    "connect",
    "ActAs",
    "Admin",
    "ArchiveEvent",
    "Boundary",
    "Command",
    "CommandMeta",
    "CreateAndExerciseCommand",
    "CreateCommand",
    "CreateEvent",
    "Event",
    "EventOrBoundary",
    "ExerciseByKeyCommand",
    "ExerciseCommand",
    "ExerciseResponse",
    "MeteringReport",
    "PartyInfo",
    "PackageService",
    "ReadAs",
    "Connection",
    "QueryStream",
    "User",
]


# noinspection PyShadowingNames
def connect(*, blocking=False, **kwargs):
    """
    Create a connection from the supplied parameters.

    See the `documentation for this function
    <https://digital-asset.github.io/dazl-client/dazl.ledger.html#dazl.ledger.connect>`_ for more
    details on the parameters it takes and how values are defaulted.
    """
    from .config import Config
    from .grpc.conn_aio import Connection as GrpcConnection

    if blocking:
        from .blocking._aiowrapper import ConnectionThunk

        return ConnectionThunk(lambda: GrpcConnection(Config.create(**kwargs)))
    else:
        cfg = Config.create(**kwargs)
        conn = GrpcConnection(cfg)

    return conn


class PackageService(Protocol):
    """
    Protocol that describe a service that provides package information. The :class:`Connection`
    protocol extends this interface.
    """

    def get_package(self, package_id, /, *, timeout=None):
        """
        Given a package ID, fetch the binary data for the corresponding DALF.

        :param package_id: The package ID of the DALF to retrieve.
        :param timeout: The maximum length of time to wait before giving up.
        :return: The byte array contents of the DALF associated with the package ID.
        """
        raise NotImplementedError

    def list_package_ids(self, *, timeout=None):
        """
        Fetch a list of all known package IDs.

        :param timeout: The maximum length of time to wait before giving up.
        """
        raise NotImplementedError


@runtime_checkable
class Connection(PackageService, Protocol):
    """
    Protocol that describes a connection to a ledger. You will typically work with the more specific
    protocols :class:`dazl.ledger.aio.Connection` or :class:`dazl.ledger.blocking.Connection` that
    are tailored towards connections with ``asyncio`` or thread-blocking semantics, respectively.
    """

    @property
    def config(self):
        """
        Get the configuration that created this connection.

        Under certain circumstances (notably, OAuth token refreshing), fields of this configuration
        object are modifiable after the connection has been created; see the documentation for
        :class:`dazl.ledger.config.Config` for more information.
        """
        raise NotImplementedError

    @property
    def codec(self):
        """
        Get the object that provides conversion to over-the-wire types for this connection.

        The specific object here is transport-specific.
        """
        raise NotImplementedError

    @property
    def is_closed(self):
        """
        Return whether the connection is closed.
        """
        raise NotImplementedError

    def open(self):
        """
        Prepares the connection for being used, including possibly fetching the ledger ID if it is
        not yet known.

        On *asynchronous* connections, this method is a coroutine.

        On *blocking* connections, this method blocks the current thread until a connection is
        ready.
        """
        raise NotImplementedError

    def close(self):
        """
        Close the connection.

        On _asynchronous_ connections, this method is a coroutine.

        On _blocking_ connections, this method blocks the current thread until the connection is
        closed and all network activity has stopped.
        """
        raise NotImplementedError

    def submit(
        self,
        commands,
        /,
        *,
        workflow_id=None,
        command_id=None,
        read_as=None,
        act_as=None,
    ):
        """
        Submit one or more commands to the Ledger API.

        You should generally prefer trying to use :meth:`create`, :meth:`exercise`,
        :meth:`exercise_by_key`, or :meth:`create_and_exercise`, as they are available over both
        the gRPC Ledger API and HTTP JSON API; additionally those methods can provide more
        information about what happened.

        This method can be used to submit multiple disparate commands as a single transaction, but
        if you find yourself needing to do this, you may want to consider moving more of your logic
        into Daml so that only a single command is needed from the outside in order to satisfy your
        use case.

        On *asynchronous* connections, this method is a coroutine.

        On *blocking* connections, this method blocks the current thread until the connection is
        closed and all network activity has stopped.

        :param commands:
            A command or sequence of commands to submit to the ledger.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        """
        raise NotImplementedError

    def create(
        self,
        template_id,
        payload,
        /,
        *,
        workflow_id=None,
        command_id=None,
        read_as=None,
        act_as=None,
    ):
        """
        Create a contract for a given template.

        :param template_id:
            The template of the contract to be created.

            Note that future versions reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param payload:
            Template arguments for the contract to be created.

            Note that future versions reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :return:
            The :class:`CreateEvent` that represents the contract that was successfully created.
        """
        raise NotImplementedError

    def exercise(
        self,
        contract_id,
        choice_name,
        argument,
        /,
        *,
        workflow_id=None,
        command_id=None,
        read_as=None,
        act_as=None,
    ):
        """
        Exercise a choice on a contract identified by its contract ID.

        :param contract_id:
            The contract ID of the contract to exercise.
        :param choice_name:
            The name of the choice to exercise.
        :param argument:
            The choice arguments. Can be omitted for choices that take no argument.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        raise NotImplementedError

    def create_and_exercise(
        self,
        template_id,
        payload,
        choice_name,
        argument=None,
        /,
        *,
        workflow_id=None,
        command_id=None,
        read_as=None,
        act_as=None,
    ):
        """
        Exercise a choice on a newly-created contract, in a single transaction.

        :param template_id:
            The template of the contract to be created (positional argument only).
        :param payload:
            Template arguments for the contract to be created (positional argument only).
        :param choice_name:
            The name of the choice to exercise (positional argument only).
        :param argument:
            The choice arguments. Can be omitted for choices that take no argument (positional
            argument only).
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        raise NotImplementedError

    def exercise_by_key(
        self,
        template_id,
        choice_name,
        key,
        argument,
        /,
        *,
        workflow_id=None,
        command_id=None,
        read_as=None,
        act_as=None,
    ):
        """
        Exercise a choice on a contract identified by its contract key.

        :param template_id:
            The template of the contract to be created (positional argument only).
        :param choice_name:
            The name of the choice to exercise.
        :param key:
            The key of the contract to exercise.
        :param argument:
            The choice arguments. Can be omitted for choices that take no argument.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        raise NotImplementedError

    def archive(
        self,
        contract_id,
        /,
        *,
        workflow_id=None,
        command_id=None,
        read_as=None,
        act_as=None,
    ):
        """
        Archive a choice on a contract identified by its contract ID.

        :param contract_id:
            The contract ID of the contract to exercise.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        raise NotImplementedError

    def archive_by_key(
        self,
        template_id,
        key,
        /,
        *,
        workflow_id=None,
        command_id=None,
        read_as=None,
        act_as=None,
    ):
        """
        Exercise a choice on a contract identified by its contract key.

        :param template_id:
            The template of the contract to be created (positional argument only).
        :param key:
            The key of the contract to exercise.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        raise NotImplementedError

    def query(self, template_id="*", query=None, /, *, read_as=None):
        """
        Return the create events from the active contract set service as a stream.

        If you find yourself repeatedly calling :meth:`query` or :meth:`query_many` over the same
        set of templates, you may want to consider :class:`ACS` instead, which is a utility class
        that helps you maintain a "live" state of the ACS.

        :param template_id:
            The name of the template for which to fetch contracts.
        :param query:
            A filter to apply to the set of returned contracts.
        :param read_as:
            An optional set of read-as parties to use to submit this query. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :return:
            A query stream that iterates over the :class:`CreateEvent` objects of the Active
            Contract Set, and terminates.
        """
        raise NotImplementedError

    def query_many(self, *q, read_as=None):
        """
        Return the create events from the active contract set service as a stream.

        If you find yourself repeatedly calling :meth:`query` or :meth:`query_many` over the same
        set of templates, you may want to consider :class:`ACS` instead, which is a utility class
        that helps you maintain a "live" state of the ACS.

        Examples:

            # Return a stream that returns a snapshot of the complete ACS
            stream = conn.query_many()

            # Return a stream that iterates over the Magic template in the So.Much module
            stream = conn.query_many("So.Much:Magic")

            # Return a stream that iterates over two templates
            # (can either be called with multiple parameters or as a single list)
            stream = conn.query_many("So.Much:Magic", "So.Much:Awesome")
            stream = conn.query_many(["So.Much:Magic", "So.Much:Awesome"])

            # Return a stream that iterates over Magic templates
            # with a field `power` having a value of 100
            stream = conn.query_many({"So.Much:Magic": {"power": 100}})
        """
        raise NotImplementedError

    def stream(self, template_id="*", query=None, /, *, offset=None, read_as=None):
        """
        Stream create/archive events.

        When ``offset`` is ``None``, create events from the active contract set are returned first,
        followed by a continuous stream of updates (creates/archives).

        Otherwise, ``offset`` can be supplied to resume a stream from a prior point where a
        ``Boundary`` was returned from a previous stream.

        :param template_id:
            The name of the template for which to fetch contracts.
        :param query:
            A filter to apply to the set of returned contracts. Note that this does not filter
            :class:`ArchiveEvent`; readers of the stream MUST be able to cope with "mismatched"
            archives that come from the result of applying a filter.
        :param offset:
            An optional offset at which to start receiving events. If ``None``, start from the
            beginning.
        :param read_as:
            An optional set of read-as parties to use to submit this query. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        """
        raise NotImplementedError

    def stream_many(self, *q, offset=None, read_as=None):
        """
        Stream create/archive events from more than one template ID in the same stream.

        When ``offset`` is ``None``, create events from the active contract set are returned first,
        followed by a continuous stream of updates (creates/archives).

        Otherwise, ``offset`` can be supplied to resume a stream from a prior point where a
        ``Boundary`` was returned from a previous stream.

        Examples:

            # Return a stream that returns a snapshot of the complete ACS
            stream = conn.stream_many()

            # Return a stream that iterates over the Magic template in the So.Much module
            stream = conn.stream_many("So.Much:Magic")

            # Return a stream that iterates over two templates
            # (can either be called with multiple parameters or as a single list)
            stream = conn.stream_many("So.Much:Magic", "So.Much:Awesome")
            stream = conn.stream_many(["So.Much:Magic", "So.Much:Awesome"])

            # Return a stream that iterates over Magic templates
            # with a field `power` having a value of 100
            stream = conn.stream_many({"So.Much:Magic": {"power": 100}})
        """
        raise NotImplementedError

    def get_user(self, user_id=None, /):
        """
        Get the user data of a specific user or the authenticated user.

        :param user_id:
            If supplied, the user whose data to retrieve. If unspecified, then the data for the
            currently authenticated user will be retrieved.
        :return:
            Information about the specified :class:`User`.
        """
        raise NotImplementedError

    def list_users(self):
        """
        List all existing users.

        :return:
            Information about all :class:`User`'s on the backing participant.
        """
        raise NotImplementedError

    def allocate_party(self, *, identifier_hint=None, display_name=None):
        """
        Allocate a new party.

        This method requires that the connection be created with ``admin=true``, or a token that
        contains an ``admin=true`` claim.

        :param identifier_hint:
            A hint to the backing participant. Note that the Daml ledger is free to ignore this
            hint entirely.
        :type identifier_hint: str
        :param display_name:
            A human-readable name that corresponds to the allocated ``Party``.
        :type display_name: str
        :return:
            Information about the allocated ``Party``.
        """
        raise NotImplementedError

    def list_known_parties(self):
        """
        Return a list of :class:`PartyInfo` for all parties on the ledger.
        """
        raise NotImplementedError

    def upload_package(self, contents, /):
        """
        Upload a byte array as a DAR file to the remote Daml ledger.

        This method requires that the connection be created with ``admin=true``, or a token that
        contains an ``admin=true`` claim.

        :param contents:
            The contents of a DAR file (for example, as obtained from running ``daml build``).
        """
        raise NotImplementedError

    def get_metering_report(self, from_, to=None, application_id=None):
        """
        Retrieve a metering report.

        :param from_:
            The from timestamp (inclusive).
        :param to:
            If specified, the to timestamp (exclusive). If not provided, the server will default to
            its current time.
        :param application_id:
            If specified, the report will only be generated for the specified application.
        :return:
            A :class:`ParticipantMeteringReport`.
        """
        raise NotImplementedError


class QueryStream(Protocol):
    """
    Protocol for classes that provide for reading from a stream of events from a Daml ledger. Like
    :class:`Connection`, there are async query streams (:class:`dazl.ledger.aio.QueryStream`) and
    blocking query streams (:class:`dazl.ledger.blocking.QueryStream`).
    """

    def on_create(self, *args):
        """
        Register a callback that is triggered whenever a :class:`CreateEvent` is read through the
        stream.
        """
        raise NotImplementedError

    def on_archive(self, *args):
        """
        Register a callback that is triggered whenever a :class:`ArchiveEvent` is read through the
        stream.
        """
        raise NotImplementedError

    def on_boundary(self, *args):
        """
        Register a callback that is triggered whenever a :class:`Boundary` is read through the
        stream.
        """
        raise NotImplementedError

    def close(self):
        """
        Close and dispose of any resources used by this stream.

        This can be called at any time to end a stream prematurely, and is the only way to abort
        a stream returned by :meth:`Connection.stream`/:meth:`Connection.stream_many`.
        """
        raise NotImplementedError

    def run(self):
        """
        Consume the stream without returning any values.

        This method is useful if you use :meth:`on_create`, :meth:`on_archive`, or
        :meth:`on_boundary` to register callbacks for events.

        On _asynchronous_ connections, this method is a coroutine that terminates once the stream
        has ended. For streams returned by :meth:`Connection.stream`/:meth:`Connection.stream_many`,
        the stream is only ended by :meth:`close`.

        On _blocking_ connections, this method blocks until the stream has ended.

        For streams returned by :meth:`Connection.stream`/:meth:`Connection.stream_many`, the
        method blocks indefinitely, and is only closed if :meth:`close` is called or the
        server disconnects.
        """
        raise NotImplementedError

    def creates(self):
        """
        Return a stream of :class:`CreateEvent` objects. This will include the contracts of the
        Active Contract Set, as well as create events over subsequent transactions.

        On _asynchronous_ connections, this method is an asynchronous iterator: use ``async for``
        to iterate over its contents.

        On _blocking_ connections, this method a blocking iterator: use ``for`` to iterate over its
        contents.

        For streams returned by :meth:`Connection.stream`/:meth:`Connection.stream_many`, the
        iterator remains open indefinitely, and is only closed if :meth:`close` is called or the
        server disconnects.
        """
        raise NotImplementedError

    def events(self):
        """
        Return a stream of :class:`CreateEvent` and :class:`ArchiveEvent` objects. This will include
        the initial contracts of the Active Contract Set, as well as create and archive events over
        subsequent transactions. You may see :class:`ArchiveEvent` objects that had no
        :class:`CreateEvent`s predecessor; this will typically happen with event filtering (which
        can only effectively filter over :class:`CreateEvent`

        On _asynchronous_ connections, this method is an asynchronous iterator: use ``async for``
        to iterate over its contents.

        On _blocking_ connections, this method a blocking iterator: use ``for`` to iterate over its
        contents.

        For streams returned by :meth:`Connection.stream`/:meth:`Connection.stream_many`, the
        iterator remains open indefinitely, and is only closed if :meth:`close` is called or the
        server disconnects.
        """
        raise NotImplementedError

    def items(self):
        """
        Return a stream of :class:`CreateEvent`, :class:`ArchiveEvent`, and :class:`Boundary`
        objects. This will include the initial contracts of the Active Contract Set, as well as
        create and archive events over subsequent transactions, and :class:`Boundary` objects that
        describe offsets where an interrupted stream can be resumed without having to re-read the
        initial contracts of the Active Contract Set.

        On _asynchronous_ connections, this method is an asynchronous iterator: use ``async for``
        to iterate over its contents.

        On _blocking_ connections, this method a blocking iterator: use ``for`` to iterate over its
        contents.

        For streams returned by :meth:`Connection.stream`/:meth:`Connection.stream_many`, the
        iterator remains open indefinitely, and is only closed if :meth:`close` is called or the
        server disconnects.
        """
        raise NotImplementedError



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

HEADER = """
// Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
// SPDX-License-Identifier: Apache-2.0
""".lstrip()



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.message import Message as _Message
from google.protobuf.struct_pb2 import Struct
from google.protobuf.timestamp_pb2 import Timestamp

__all__ = [
    "GetMeteringReportRequest",
    "GetMeteringReportResponse",
]


class GetMeteringReportRequest(_Message):
    @property
    def to(self) -> Timestamp: ...
    application_id: _builtins.str
    def __init__(self, *, to: _typing.Optional[Timestamp] = ..., application_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["from", "to", "application_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["from", "to", "application_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetMeteringReportResponse(_Message):
    @property
    def request(self) -> GetMeteringReportRequest: ...
    @property
    def report_generation_time(self) -> Timestamp: ...
    @property
    def metering_report_json(self) -> Struct: ...
    def __init__(self, *, request: _typing.Optional[GetMeteringReportRequest] = ..., report_generation_time: _typing.Optional[Timestamp] = ..., metering_report_json: _typing.Optional[Struct] = ...): ...
    def HasField(self, field_name: _typing.Literal["request", "report_generation_time", "metering_report_json"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["request", "report_generation_time", "metering_report_json"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import identity_provider_config_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2


class IdentityProviderConfigServiceStub(object):
    """Identity Provider Config Service makes it possible for participant node administrators
    to setup and manage additional identity providers at runtime.

    This allows using access tokens from identity providers unknown at deployment time. When an identity
    provider is configured, independent IDP administrators can manage their own set of parties and users.
    Such parties and users have a matching `identity_provider_id` defined and are inaccessible to
    administrators from other identity providers. A user will only be authenticated if the corresponding JWT
    token is issued by the appropriate identity provider.
    Users and parties without `identity_provider_id` defined are assumed to be using the default identity provider,
    which is configured statically at the participant node's deployment time.

    The Ledger API uses the "iss" claim of a JWT token to match the token to a specific IDP. If there is no match,
    the default IDP is assumed.

    The fields of request messages (and sub-messages) are marked either as ``Optional`` or ``Required``:
    (1) ``Optional`` denoting the client may leave the field unset when sending a request.
    (2) ``Required`` denoting the client must set the field to a non-default value when sending a request.

    An identity provider config resource is described by the ``IdentityProviderConfig`` message,
    An identity provider config resource, once it has been created, can be modified.
    In order to update the properties represented by the ``IdentityProviderConfig`` message use the ``UpdateIdentityProviderConfig`` RPC.
    The only fields that can be modified are those marked as ``Modifiable``.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateIdentityProviderConfig = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/CreateIdentityProviderConfig',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.CreateIdentityProviderConfigRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.CreateIdentityProviderConfigResponse.FromString,
                )
        self.GetIdentityProviderConfig = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/GetIdentityProviderConfig',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.GetIdentityProviderConfigRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.GetIdentityProviderConfigResponse.FromString,
                )
        self.UpdateIdentityProviderConfig = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/UpdateIdentityProviderConfig',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.UpdateIdentityProviderConfigRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.UpdateIdentityProviderConfigResponse.FromString,
                )
        self.ListIdentityProviderConfigs = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/ListIdentityProviderConfigs',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.ListIdentityProviderConfigsRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.ListIdentityProviderConfigsResponse.FromString,
                )
        self.DeleteIdentityProviderConfig = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/DeleteIdentityProviderConfig',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.DeleteIdentityProviderConfigRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.DeleteIdentityProviderConfigResponse.FromString,
                )


class IdentityProviderConfigServiceServicer(object):
    """Identity Provider Config Service makes it possible for participant node administrators
    to setup and manage additional identity providers at runtime.

    This allows using access tokens from identity providers unknown at deployment time. When an identity
    provider is configured, independent IDP administrators can manage their own set of parties and users.
    Such parties and users have a matching `identity_provider_id` defined and are inaccessible to
    administrators from other identity providers. A user will only be authenticated if the corresponding JWT
    token is issued by the appropriate identity provider.
    Users and parties without `identity_provider_id` defined are assumed to be using the default identity provider,
    which is configured statically at the participant node's deployment time.

    The Ledger API uses the "iss" claim of a JWT token to match the token to a specific IDP. If there is no match,
    the default IDP is assumed.

    The fields of request messages (and sub-messages) are marked either as ``Optional`` or ``Required``:
    (1) ``Optional`` denoting the client may leave the field unset when sending a request.
    (2) ``Required`` denoting the client must set the field to a non-default value when sending a request.

    An identity provider config resource is described by the ``IdentityProviderConfig`` message,
    An identity provider config resource, once it has been created, can be modified.
    In order to update the properties represented by the ``IdentityProviderConfig`` message use the ``UpdateIdentityProviderConfig`` RPC.
    The only fields that can be modified are those marked as ``Modifiable``.
    """

    def CreateIdentityProviderConfig(self, request, context):
        """Create a new identity provider configuration.
        The request will fail if the maximum allowed number of separate configurations is reached.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIdentityProviderConfig(self, request, context):
        """Get the identity provider configuration data by id.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateIdentityProviderConfig(self, request, context):
        """Update selected modifiable attribute of an identity provider config resource described
        by the ``IdentityProviderConfig`` message.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListIdentityProviderConfigs(self, request, context):
        """List all existing identity provider configurations.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteIdentityProviderConfig(self, request, context):
        """Delete an existing identity provider configuration.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_IdentityProviderConfigServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateIdentityProviderConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateIdentityProviderConfig,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.CreateIdentityProviderConfigRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.CreateIdentityProviderConfigResponse.SerializeToString,
            ),
            'GetIdentityProviderConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIdentityProviderConfig,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.GetIdentityProviderConfigRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.GetIdentityProviderConfigResponse.SerializeToString,
            ),
            'UpdateIdentityProviderConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateIdentityProviderConfig,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.UpdateIdentityProviderConfigRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.UpdateIdentityProviderConfigResponse.SerializeToString,
            ),
            'ListIdentityProviderConfigs': grpc.unary_unary_rpc_method_handler(
                    servicer.ListIdentityProviderConfigs,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.ListIdentityProviderConfigsRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.ListIdentityProviderConfigsResponse.SerializeToString,
            ),
            'DeleteIdentityProviderConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteIdentityProviderConfig,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.DeleteIdentityProviderConfigRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.DeleteIdentityProviderConfigResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.daml.ledger.api.v1.admin.IdentityProviderConfigService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class IdentityProviderConfigService(object):
    """Identity Provider Config Service makes it possible for participant node administrators
    to setup and manage additional identity providers at runtime.

    This allows using access tokens from identity providers unknown at deployment time. When an identity
    provider is configured, independent IDP administrators can manage their own set of parties and users.
    Such parties and users have a matching `identity_provider_id` defined and are inaccessible to
    administrators from other identity providers. A user will only be authenticated if the corresponding JWT
    token is issued by the appropriate identity provider.
    Users and parties without `identity_provider_id` defined are assumed to be using the default identity provider,
    which is configured statically at the participant node's deployment time.

    The Ledger API uses the "iss" claim of a JWT token to match the token to a specific IDP. If there is no match,
    the default IDP is assumed.

    The fields of request messages (and sub-messages) are marked either as ``Optional`` or ``Required``:
    (1) ``Optional`` denoting the client may leave the field unset when sending a request.
    (2) ``Required`` denoting the client must set the field to a non-default value when sending a request.

    An identity provider config resource is described by the ``IdentityProviderConfig`` message,
    An identity provider config resource, once it has been created, can be modified.
    In order to update the properties represented by the ``IdentityProviderConfig`` message use the ``UpdateIdentityProviderConfig`` RPC.
    The only fields that can be modified are those marked as ``Modifiable``.
    """

    @staticmethod
    def CreateIdentityProviderConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/CreateIdentityProviderConfig',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.CreateIdentityProviderConfigRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.CreateIdentityProviderConfigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIdentityProviderConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/GetIdentityProviderConfig',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.GetIdentityProviderConfigRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.GetIdentityProviderConfigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateIdentityProviderConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/UpdateIdentityProviderConfig',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.UpdateIdentityProviderConfigRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.UpdateIdentityProviderConfigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListIdentityProviderConfigs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/ListIdentityProviderConfigs',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.ListIdentityProviderConfigsRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.ListIdentityProviderConfigsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteIdentityProviderConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.IdentityProviderConfigService/DeleteIdentityProviderConfig',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.DeleteIdentityProviderConfigRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_identity__provider__config__service__pb2.DeleteIdentityProviderConfigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Define a few DAML values and functions that have been inlined in Python for efficiency and
simplicity.
"""

from __future__ import annotations

from typing import Any, Optional, Sequence
import warnings

from . import daml_types as daml
from .daml_lf_1 import BuiltinFunction, Expr, PrimLit, Type

warnings.warn(
    "The symbols in dazl.damlast.builtin are deprecated", DeprecationWarning, stacklevel=2
)

with warnings.catch_warnings():
    warnings.simplefilter("ignore", DeprecationWarning)
    from ._builtins_meta import Builtin, BuiltinTable

    builtins = BuiltinTable()


@builtins.add
class Concat(Builtin):
    name = "DA.Internal.Prelude:concat"
    args = (daml.List(daml.List(daml.var("x"))),)

    def evaluate(self, _: Sequence[Type], args: Sequence[Any]) -> Any:
        xxs = args[0]

        result = []
        for xs in xxs:
            result.extend(xs)
        return result

    def simplify(self, _: Sequence[Type], args: Sequence[Expr]) -> Optional[Expr]:
        xxs = args[0]

        # if there is a single-element list as an arg to concat, simply remove the function call
        # and fully unwrap
        if xxs.cons is not None and xxs.cons.tail.nil is not None and len(xxs.cons.front) == 1:
            return xxs.cons.front[0]
        if xxs.nil is not None:
            return Expr(nil=Expr.Nil(xxs.nil.type.prim.args[0]))
        return None


@builtins.add
class Show(Builtin):
    name = "GHC.Show:show"
    args = (daml.Text, daml.Text)

    def evaluate(self, _: "Sequence[Type]", args: "Sequence[Any]") -> "Any":
        return str(args[1])

    def simplify(self, _: "Sequence[Type]", args: "Sequence[Expr]") -> "Optional[Expr]":
        return None


@builtins.add
class AppendText(Builtin):
    builtin = BuiltinFunction.APPEND_TEXT
    args = (daml.Text, daml.Text)

    def evaluate(self, _: "Sequence[Type]", args: "Sequence[Any]") -> "Any":
        return args[0] + args[1]

    def simplify(self, _: "Sequence[Type]", args: "Sequence[Expr]") -> "Optional[Expr]":
        if args[0].prim_lit is not None and args[1].prim_lit is not None:
            return Expr(
                prim_lit=PrimLit(text=(args[0].prim_lit.text or "") + (args[1].prim_lit.text or ""))
            )
        return None



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.message import Message as _Message
from google.protobuf.timestamp_pb2 import Timestamp

__all__ = [
    "ContractMetadata",
]


class ContractMetadata(_Message):
    @property
    def created_at(self) -> Timestamp: ...
    contract_key_hash: _builtins.bytes
    driver_metadata: _builtins.bytes
    def __init__(self, *, created_at: _typing.Optional[Timestamp] = ..., contract_key_hash: _typing.Optional[_builtins.bytes] = ..., driver_metadata: _typing.Optional[_builtins.bytes] = ...): ...
    def HasField(self, field_name: _typing.Literal["created_at", "contract_key_hash", "driver_metadata"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["created_at", "contract_key_hash", "driver_metadata"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Any

from ..damlast.daml_lf_1 import DefDataType, Type as DamlType
from .context import Context
from .mapper import ValueMapper


class PythonSampleEncoder(ValueMapper):
    def __init__(self, max_depth: int = 6):
        self.max_depth = max_depth

    def _recurse(self, context: Context, key, item_type):
        if context.depth <= self.max_depth:
            return "..."
        else:
            return context.append_path(key).convert(item_type, None)

    def data_record(
        self, context: Context, dt: DefDataType, record: DefDataType.Fields, obj: Any
    ) -> Any:
        return {fld.field: self._recurse(context, fld.field, fld.type) for fld in record.fields}
        pass

    def data_variant(
        self, context: Context, dt: DefDataType, variant: DefDataType.Fields, obj: Any
    ) -> Any:
        return {fld.field: self._recurse(context, fld.field, fld.type) for fld in variant.fields}

    def data_enum(
        self,
        context: Context,
        dt: DefDataType,
        enum: DefDataType.EnumConstructors,
        obj: Any,
    ) -> Any:
        return " | ".join(enum.constructors)

    def prim_unit(self, context: Context, obj: Any) -> Any:
        return "{}"

    def prim_bool(self, context: Context, obj: Any) -> Any:
        return "true | false"

    def prim_int64(self, context: Context, obj: Any) -> Any:
        return "int-value"

    def prim_text(self, context: Context, obj: Any) -> Any:
        return "text-value"

    def prim_timestamp(self, context: Context, obj: Any) -> Any:
        return "timestamp-value"

    def prim_party(self, context: Context, obj: Any) -> Any:
        return "party-value"

    def prim_list(self, context: Context, item_type: DamlType, obj: Any) -> Any:
        return [self._recurse(context, "[]", item_type)]

    def prim_date(self, context: Context, obj: Any) -> Any:
        return "date-value"

    def prim_contract_id(self, context: Context, item_type: DamlType, obj: Any) -> Any:
        return "contract-id"

    def prim_optional(self, context: Context, item_type: DamlType, obj: Any) -> Any:
        return ""

    def prim_text_map(self, context: Context, item_type: DamlType, obj: Any) -> Any:
        return [self._recurse(context, "[]", item_type)]

    def prim_numeric(self, context: Context, nat: int, obj: Any) -> Any:
        pass

    def prim_gen_map(
        self, context: Context, key_type: DamlType, value_type: DamlType, obj: Any
    ) -> Any:
        return {self._recurse(context, "[]", key_type): self._recurse(context, "[]", value_type)}



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.any_pb2 import Any
from google.protobuf.duration_pb2 import Duration
from google.protobuf.internal.containers import RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
from google.protobuf.message import Message as _Message
from google.protobuf.timestamp_pb2 import Timestamp

from .contract_metadata_pb2 import ContractMetadata
from .value_pb2 import Identifier, Record, Value

__all__ = [
    "Commands",
    "Command",
    "CreateCommand",
    "ExerciseCommand",
    "ExerciseByKeyCommand",
    "CreateAndExerciseCommand",
    "DisclosedContract",
]


class Commands(_Message):
    ledger_id: _builtins.str
    workflow_id: _builtins.str
    application_id: _builtins.str
    command_id: _builtins.str
    party: _builtins.str
    @property
    def commands(self) -> RepeatedCompositeFieldContainer[Command]: ...
    @property
    def deduplication_time(self) -> Duration: ...
    @property
    def deduplication_duration(self) -> Duration: ...
    deduplication_offset: _builtins.str
    @property
    def min_ledger_time_abs(self) -> Timestamp: ...
    @property
    def min_ledger_time_rel(self) -> Duration: ...
    @property
    def act_as(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    @property
    def read_as(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    submission_id: _builtins.str
    @property
    def disclosed_contracts(self) -> RepeatedCompositeFieldContainer[DisclosedContract]: ...
    @_typing.overload
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ..., workflow_id: _typing.Optional[_builtins.str] = ..., application_id: _typing.Optional[_builtins.str] = ..., command_id: _typing.Optional[_builtins.str] = ..., party: _typing.Optional[_builtins.str] = ..., commands: _typing.Optional[_typing.Iterable[Command]] = ..., min_ledger_time_abs: _typing.Optional[Timestamp] = ..., min_ledger_time_rel: _typing.Optional[Duration] = ..., act_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., read_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., submission_id: _typing.Optional[_builtins.str] = ..., disclosed_contracts: _typing.Optional[_typing.Iterable[DisclosedContract]] = ...): ...
    @_typing.overload
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ..., workflow_id: _typing.Optional[_builtins.str] = ..., application_id: _typing.Optional[_builtins.str] = ..., command_id: _typing.Optional[_builtins.str] = ..., party: _typing.Optional[_builtins.str] = ..., commands: _typing.Optional[_typing.Iterable[Command]] = ..., deduplication_time: Duration = ..., min_ledger_time_abs: _typing.Optional[Timestamp] = ..., min_ledger_time_rel: _typing.Optional[Duration] = ..., act_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., read_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., submission_id: _typing.Optional[_builtins.str] = ..., disclosed_contracts: _typing.Optional[_typing.Iterable[DisclosedContract]] = ...): ...
    @_typing.overload
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ..., workflow_id: _typing.Optional[_builtins.str] = ..., application_id: _typing.Optional[_builtins.str] = ..., command_id: _typing.Optional[_builtins.str] = ..., party: _typing.Optional[_builtins.str] = ..., commands: _typing.Optional[_typing.Iterable[Command]] = ..., deduplication_duration: Duration = ..., min_ledger_time_abs: _typing.Optional[Timestamp] = ..., min_ledger_time_rel: _typing.Optional[Duration] = ..., act_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., read_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., submission_id: _typing.Optional[_builtins.str] = ..., disclosed_contracts: _typing.Optional[_typing.Iterable[DisclosedContract]] = ...): ...
    @_typing.overload
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ..., workflow_id: _typing.Optional[_builtins.str] = ..., application_id: _typing.Optional[_builtins.str] = ..., command_id: _typing.Optional[_builtins.str] = ..., party: _typing.Optional[_builtins.str] = ..., commands: _typing.Optional[_typing.Iterable[Command]] = ..., deduplication_offset: _builtins.str = ..., min_ledger_time_abs: _typing.Optional[Timestamp] = ..., min_ledger_time_rel: _typing.Optional[Duration] = ..., act_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., read_as: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., submission_id: _typing.Optional[_builtins.str] = ..., disclosed_contracts: _typing.Optional[_typing.Iterable[DisclosedContract]] = ...): ...
    def HasField(self, field_name: _typing.Literal["ledger_id", "workflow_id", "application_id", "command_id", "party", "commands", "deduplication_period", "deduplication_time", "deduplication_duration", "deduplication_offset", "min_ledger_time_abs", "min_ledger_time_rel", "act_as", "read_as", "submission_id", "disclosed_contracts"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["ledger_id", "workflow_id", "application_id", "command_id", "party", "commands", "deduplication_period", "deduplication_time", "deduplication_duration", "deduplication_offset", "min_ledger_time_abs", "min_ledger_time_rel", "act_as", "read_as", "submission_id", "disclosed_contracts"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.Literal["deduplication_period"]) -> _typing.Literal[None, "deduplication_time", "deduplication_duration", "deduplication_offset"]: ...

class Command(_Message):
    @property
    def create(self) -> CreateCommand: ...
    @property
    def exercise(self) -> ExerciseCommand: ...
    @property
    def exerciseByKey(self) -> ExerciseByKeyCommand: ...
    @property
    def createAndExercise(self) -> CreateAndExerciseCommand: ...
    @_typing.overload
    def __init__(self): ...
    @_typing.overload
    def __init__(self, *, create: CreateCommand = ...): ...
    @_typing.overload
    def __init__(self, *, exercise: ExerciseCommand = ...): ...
    @_typing.overload
    def __init__(self, *, exerciseByKey: ExerciseByKeyCommand = ...): ...
    @_typing.overload
    def __init__(self, *, createAndExercise: CreateAndExerciseCommand = ...): ...
    def HasField(self, field_name: _typing.Literal["command", "create", "exercise", "exerciseByKey", "createAndExercise"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["command", "create", "exercise", "exerciseByKey", "createAndExercise"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.Literal["command"]) -> _typing.Literal[None, "create", "exercise", "exerciseByKey", "createAndExercise"]: ...

class CreateCommand(_Message):
    @property
    def template_id(self) -> Identifier: ...
    @property
    def create_arguments(self) -> Record: ...
    def __init__(self, *, template_id: _typing.Optional[Identifier] = ..., create_arguments: _typing.Optional[Record] = ...): ...
    def HasField(self, field_name: _typing.Literal["template_id", "create_arguments"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["template_id", "create_arguments"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ExerciseCommand(_Message):
    @property
    def template_id(self) -> Identifier: ...
    contract_id: _builtins.str
    choice: _builtins.str
    @property
    def choice_argument(self) -> Value: ...
    def __init__(self, *, template_id: _typing.Optional[Identifier] = ..., contract_id: _typing.Optional[_builtins.str] = ..., choice: _typing.Optional[_builtins.str] = ..., choice_argument: _typing.Optional[Value] = ...): ...
    def HasField(self, field_name: _typing.Literal["template_id", "contract_id", "choice", "choice_argument"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["template_id", "contract_id", "choice", "choice_argument"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ExerciseByKeyCommand(_Message):
    @property
    def template_id(self) -> Identifier: ...
    @property
    def contract_key(self) -> Value: ...
    choice: _builtins.str
    @property
    def choice_argument(self) -> Value: ...
    def __init__(self, *, template_id: _typing.Optional[Identifier] = ..., contract_key: _typing.Optional[Value] = ..., choice: _typing.Optional[_builtins.str] = ..., choice_argument: _typing.Optional[Value] = ...): ...
    def HasField(self, field_name: _typing.Literal["template_id", "contract_key", "choice", "choice_argument"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["template_id", "contract_key", "choice", "choice_argument"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class CreateAndExerciseCommand(_Message):
    @property
    def template_id(self) -> Identifier: ...
    @property
    def create_arguments(self) -> Record: ...
    choice: _builtins.str
    @property
    def choice_argument(self) -> Value: ...
    def __init__(self, *, template_id: _typing.Optional[Identifier] = ..., create_arguments: _typing.Optional[Record] = ..., choice: _typing.Optional[_builtins.str] = ..., choice_argument: _typing.Optional[Value] = ...): ...
    def HasField(self, field_name: _typing.Literal["template_id", "create_arguments", "choice", "choice_argument"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["template_id", "create_arguments", "choice", "choice_argument"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class DisclosedContract(_Message):
    @property
    def template_id(self) -> Identifier: ...
    contract_id: _builtins.str
    @property
    def create_arguments(self) -> Record: ...
    @property
    def create_arguments_blob(self) -> Any: ...
    @property
    def metadata(self) -> ContractMetadata: ...
    @_typing.overload
    def __init__(self, *, template_id: _typing.Optional[Identifier] = ..., contract_id: _typing.Optional[_builtins.str] = ..., metadata: _typing.Optional[ContractMetadata] = ...): ...
    @_typing.overload
    def __init__(self, *, template_id: _typing.Optional[Identifier] = ..., contract_id: _typing.Optional[_builtins.str] = ..., create_arguments: Record = ..., metadata: _typing.Optional[ContractMetadata] = ...): ...
    @_typing.overload
    def __init__(self, *, template_id: _typing.Optional[Identifier] = ..., contract_id: _typing.Optional[_builtins.str] = ..., create_arguments_blob: Any = ..., metadata: _typing.Optional[ContractMetadata] = ...): ...
    def HasField(self, field_name: _typing.Literal["template_id", "contract_id", "arguments", "create_arguments", "create_arguments_blob", "metadata"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["template_id", "contract_id", "arguments", "create_arguments", "create_arguments_blob", "metadata"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.Literal["arguments"]) -> _typing.Literal[None, "create_arguments", "create_arguments_blob"]: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import package_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2


class PackageServiceStub(object):
    """Allows clients to query the Daml-LF packages that are supported by the server.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListPackages = channel.unary_unary(
                '/com.daml.ledger.api.v1.PackageService/ListPackages',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.ListPackagesRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.ListPackagesResponse.FromString,
                )
        self.GetPackage = channel.unary_unary(
                '/com.daml.ledger.api.v1.PackageService/GetPackage',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageResponse.FromString,
                )
        self.GetPackageStatus = channel.unary_unary(
                '/com.daml.ledger.api.v1.PackageService/GetPackageStatus',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageStatusRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageStatusResponse.FromString,
                )


class PackageServiceServicer(object):
    """Allows clients to query the Daml-LF packages that are supported by the server.
    """

    def ListPackages(self, request, context):
        """Returns the identifiers of all supported packages.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPackage(self, request, context):
        """Returns the contents of a single package.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPackageStatus(self, request, context):
        """Returns the status of a single package.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PackageServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListPackages': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPackages,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.ListPackagesRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.ListPackagesResponse.SerializeToString,
            ),
            'GetPackage': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPackage,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageResponse.SerializeToString,
            ),
            'GetPackageStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPackageStatus,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageStatusRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageStatusResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.daml.ledger.api.v1.PackageService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PackageService(object):
    """Allows clients to query the Daml-LF packages that are supported by the server.
    """

    @staticmethod
    def ListPackages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.PackageService/ListPackages',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.ListPackagesRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.ListPackagesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPackage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.PackageService/GetPackage',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPackageStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.PackageService/GetPackageStatus',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageStatusRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_package__service__pb2.GetPackageStatusResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/contract_metadata.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.com/daml/ledger/api/v1/contract_metadata.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\xa2\x01\n\x10\x43ontractMetadata\x12\x39\n\ncreated_at\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12*\n\x11\x63ontract_key_hash\x18\x02 \x01(\x0cR\x0f\x63ontractKeyHash\x12\'\n\x0f\x64river_metadata\x18\x03 \x01(\x0cR\x0e\x64riverMetadataB\x94\x01\n\x16\x63om.daml.ledger.api.v1B\x1a\x43ontractMetadataOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.daml.ledger.api.v1.contract_metadata_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\032ContractMetadataOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _globals['_CONTRACTMETADATA']._serialized_start=108
  _globals['_CONTRACTMETADATA']._serialized_end=270
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.field_mask_pb2 import FieldMask
from google.protobuf.internal.containers import RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
from google.protobuf.message import Message as _Message

from .object_meta_pb2 import ObjectMeta

__all__ = [
    "GetParticipantIdRequest",
    "GetParticipantIdResponse",
    "GetPartiesRequest",
    "GetPartiesResponse",
    "ListKnownPartiesRequest",
    "ListKnownPartiesResponse",
    "AllocatePartyRequest",
    "AllocatePartyResponse",
    "UpdatePartyDetailsRequest",
    "UpdatePartyDetailsResponse",
    "PartyDetails",
]


class GetParticipantIdRequest(_Message):
    def __init__(self): ...
    def HasField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def ClearField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetParticipantIdResponse(_Message):
    participant_id: _builtins.str
    def __init__(self, *, participant_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["participant_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["participant_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetPartiesRequest(_Message):
    @property
    def parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    identity_provider_id: _builtins.str
    def __init__(self, *, parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., identity_provider_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["parties", "identity_provider_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["parties", "identity_provider_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetPartiesResponse(_Message):
    @property
    def party_details(self) -> RepeatedCompositeFieldContainer[PartyDetails]: ...
    def __init__(self, *, party_details: _typing.Optional[_typing.Iterable[PartyDetails]] = ...): ...
    def HasField(self, field_name: _typing.Literal["party_details"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["party_details"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ListKnownPartiesRequest(_Message):
    identity_provider_id: _builtins.str
    def __init__(self, *, identity_provider_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ListKnownPartiesResponse(_Message):
    @property
    def party_details(self) -> RepeatedCompositeFieldContainer[PartyDetails]: ...
    def __init__(self, *, party_details: _typing.Optional[_typing.Iterable[PartyDetails]] = ...): ...
    def HasField(self, field_name: _typing.Literal["party_details"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["party_details"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class AllocatePartyRequest(_Message):
    party_id_hint: _builtins.str
    display_name: _builtins.str
    @property
    def local_metadata(self) -> ObjectMeta: ...
    identity_provider_id: _builtins.str
    def __init__(self, *, party_id_hint: _typing.Optional[_builtins.str] = ..., display_name: _typing.Optional[_builtins.str] = ..., local_metadata: _typing.Optional[ObjectMeta] = ..., identity_provider_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["party_id_hint", "display_name", "local_metadata", "identity_provider_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["party_id_hint", "display_name", "local_metadata", "identity_provider_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class AllocatePartyResponse(_Message):
    @property
    def party_details(self) -> PartyDetails: ...
    def __init__(self, *, party_details: _typing.Optional[PartyDetails] = ...): ...
    def HasField(self, field_name: _typing.Literal["party_details"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["party_details"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class UpdatePartyDetailsRequest(_Message):
    @property
    def party_details(self) -> PartyDetails: ...
    @property
    def update_mask(self) -> FieldMask: ...
    def __init__(self, *, party_details: _typing.Optional[PartyDetails] = ..., update_mask: _typing.Optional[FieldMask] = ...): ...
    def HasField(self, field_name: _typing.Literal["party_details", "update_mask"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["party_details", "update_mask"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class UpdatePartyDetailsResponse(_Message):
    @property
    def party_details(self) -> PartyDetails: ...
    def __init__(self, *, party_details: _typing.Optional[PartyDetails] = ...): ...
    def HasField(self, field_name: _typing.Literal["party_details"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["party_details"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class PartyDetails(_Message):
    party: _builtins.str
    display_name: _builtins.str
    is_local: _builtins.bool
    @property
    def local_metadata(self) -> ObjectMeta: ...
    identity_provider_id: _builtins.str
    def __init__(self, *, party: _typing.Optional[_builtins.str] = ..., display_name: _typing.Optional[_builtins.str] = ..., is_local: _typing.Optional[_builtins.bool] = ..., local_metadata: _typing.Optional[ObjectMeta] = ..., identity_provider_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["party", "display_name", "is_local", "local_metadata", "identity_provider_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["party", "display_name", "is_local", "local_metadata", "identity_provider_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
:mod:`dazl.pretty` package
==========================

This module contains utilities for pretty-printing various types in dazl.

"""

from __future__ import annotations

from typing import TYPE_CHECKING, Optional, Type

from .util import maybe_parentheses

if TYPE_CHECKING:
    from .pygments_daml_lexer import DAMLLexer as _DAMLLexer_TYPE


__all__ = ["DAMLLexer"]


def _import_daml_lexer() -> "Optional[Type[_DAMLLexer_TYPE]]":
    # pygments isn't absolutely required, but if it's loaded, also provide our lexer
    try:
        from .pygments_daml_lexer import DAMLLexer

        return DAMLLexer
    except ImportError:
        return None


DAMLLexer = _import_daml_lexer()



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Simple methods for instantiating an application written against dazl that incorporate common usage
patterns.
"""
from __future__ import annotations

from argparse import ArgumentParser
from typing import Callable
import warnings

from ..client.api import Network
from ..client.config import NetworkConfig, configure_parser


def run(init: "Callable[[Network], None]"):
    warnings.warn("dazl.run is deprecated", DeprecationWarning, stacklevel=2)

    if init is None:
        raise ValueError("The init callback cannot be None")

    arg_parser = ArgumentParser()

    configure_parser(arg_parser, config_file_support=True)
    args = arg_parser.parse_args()

    config = NetworkConfig.get_config(args)
    network = Network()
    network.set_config(config)

    init(network)

    network.run_forever()



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
:mod:`dazl.util` package
========================

Module that exposes general utility functions.
"""

from __future__ import annotations

from .io import find_free_port
from .proc_util import ProcessLogger, kill_process_tree, wait_for_process_port

__all__ = ["find_free_port", "ProcessLogger", "kill_process_tree", "wait_for_process_port"]



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/transaction_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import ledger_offset_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2
from . import transaction_filter_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__filter__pb2
from . import transaction_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0com/daml/ledger/api/v1/transaction_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a*com/daml/ledger/api/v1/ledger_offset.proto\x1a/com/daml/ledger/api/v1/transaction_filter.proto\x1a(com/daml/ledger/api/v1/transaction.proto\"\x86\x02\n\x16GetTransactionsRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12:\n\x05\x62\x65gin\x18\x02 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x05\x62\x65gin\x12\x36\n\x03\x65nd\x18\x03 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x03\x65nd\x12\x41\n\x06\x66ilter\x18\x04 \x01(\x0b\x32).com.daml.ledger.api.v1.TransactionFilterR\x06\x66ilter\x12\x18\n\x07verbose\x18\x05 \x01(\x08R\x07verbose\"b\n\x17GetTransactionsResponse\x12G\n\x0ctransactions\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0ctransactions\"j\n\x1bGetTransactionTreesResponse\x12K\n\x0ctransactions\x18\x01 \x03(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0ctransactions\"\x87\x01\n\x1eGetTransactionByEventIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12\x19\n\x08\x65vent_id\x18\x02 \x01(\tR\x07\x65ventId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"\x8e\x01\n\x19GetTransactionByIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12%\n\x0etransaction_id\x18\x02 \x01(\tR\rtransactionId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"c\n\x16GetTransactionResponse\x12I\n\x0btransaction\x18\x01 \x01(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0btransaction\"c\n\x1aGetFlatTransactionResponse\x12\x45\n\x0btransaction\x18\x01 \x01(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0btransaction\"2\n\x13GetLedgerEndRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\"T\n\x14GetLedgerEndResponse\x12<\n\x06offset\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x06offset\"\x1f\n\x1dGetLatestPrunedOffsetsRequest\"\x99\x02\n\x1eGetLatestPrunedOffsetsResponse\x12p\n\"participant_pruned_up_to_inclusive\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x1eparticipantPrunedUpToInclusive\x12\x84\x01\n-all_divulged_contracts_pruned_up_to_inclusive\x18\x02 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\'allDivulgedContractsPrunedUpToInclusive2\x87\x08\n\x12TransactionService\x12t\n\x0fGetTransactions\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a/.com.daml.ledger.api.v1.GetTransactionsResponse0\x01\x12|\n\x13GetTransactionTrees\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a\x33.com.daml.ledger.api.v1.GetTransactionTreesResponse0\x01\x12\x81\x01\n\x17GetTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12w\n\x12GetTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12\x89\x01\n\x1bGetFlatTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12\x7f\n\x16GetFlatTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12i\n\x0cGetLedgerEnd\x12+.com.daml.ledger.api.v1.GetLedgerEndRequest\x1a,.com.daml.ledger.api.v1.GetLedgerEndResponse\x12\x87\x01\n\x16GetLatestPrunedOffsets\x12\x35.com.daml.ledger.api.v1.GetLatestPrunedOffsetsRequest\x1a\x36.com.daml.ledger.api.v1.GetLatestPrunedOffsetsResponseB\x96\x01\n\x16\x63om.daml.ledger.api.v1B\x1cTransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.daml.ledger.api.v1.transaction_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\034TransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _globals['_GETTRANSACTIONSREQUEST']._serialized_start=212
  _globals['_GETTRANSACTIONSREQUEST']._serialized_end=474
  _globals['_GETTRANSACTIONSRESPONSE']._serialized_start=476
  _globals['_GETTRANSACTIONSRESPONSE']._serialized_end=574
  _globals['_GETTRANSACTIONTREESRESPONSE']._serialized_start=576
  _globals['_GETTRANSACTIONTREESRESPONSE']._serialized_end=682
  _globals['_GETTRANSACTIONBYEVENTIDREQUEST']._serialized_start=685
  _globals['_GETTRANSACTIONBYEVENTIDREQUEST']._serialized_end=820
  _globals['_GETTRANSACTIONBYIDREQUEST']._serialized_start=823
  _globals['_GETTRANSACTIONBYIDREQUEST']._serialized_end=965
  _globals['_GETTRANSACTIONRESPONSE']._serialized_start=967
  _globals['_GETTRANSACTIONRESPONSE']._serialized_end=1066
  _globals['_GETFLATTRANSACTIONRESPONSE']._serialized_start=1068
  _globals['_GETFLATTRANSACTIONRESPONSE']._serialized_end=1167
  _globals['_GETLEDGERENDREQUEST']._serialized_start=1169
  _globals['_GETLEDGERENDREQUEST']._serialized_end=1219
  _globals['_GETLEDGERENDRESPONSE']._serialized_start=1221
  _globals['_GETLEDGERENDRESPONSE']._serialized_end=1305
  _globals['_GETLATESTPRUNEDOFFSETSREQUEST']._serialized_start=1307
  _globals['_GETLATESTPRUNEDOFFSETSREQUEST']._serialized_end=1338
  _globals['_GETLATESTPRUNEDOFFSETSRESPONSE']._serialized_start=1341
  _globals['_GETLATESTPRUNEDOFFSETSRESPONSE']._serialized_end=1622
  _globals['_TRANSACTIONSERVICE']._serialized_start=1625
  _globals['_TRANSACTIONSERVICE']._serialized_end=2656
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, typing as _typing

import grpc as _grpc
from google.protobuf.empty_pb2 import Empty
from grpc import aio as _grpc_aio

from .command_service_pb2 import SubmitAndWaitForTransactionIdResponse, SubmitAndWaitForTransactionResponse, SubmitAndWaitForTransactionTreeResponse, SubmitAndWaitRequest

__all__ = [
    "CommandServiceStub",
]


# noinspection PyPep8Naming,DuplicatedCode
class CommandServiceStub:
    @classmethod  # type: ignore
    @_typing.overload
    def __new__(cls, channel: _grpc.Channel) -> _CommandServiceBlockingStub: ...  # type: ignore
    @classmethod  # type: ignore
    @_typing.overload
    def __new__(cls, channel: _grpc_aio.Channel) -> _CommandServiceAsyncStub: ...  # type: ignore
    def SubmitAndWait(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[Empty, _grpc_aio.UnaryUnaryCall[_typing.Any, Empty]]: ...
    def SubmitAndWaitForTransactionId(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[SubmitAndWaitForTransactionIdResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, SubmitAndWaitForTransactionIdResponse]]: ...
    def SubmitAndWaitForTransaction(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[SubmitAndWaitForTransactionResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, SubmitAndWaitForTransactionResponse]]: ...
    def SubmitAndWaitForTransactionTree(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[SubmitAndWaitForTransactionTreeResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, SubmitAndWaitForTransactionTreeResponse]]: ...

# noinspection PyPep8Naming,DuplicatedCode
class _CommandServiceBlockingStub(CommandServiceStub):
    def SubmitAndWait(self, __1: SubmitAndWaitRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> Empty: ...
    def SubmitAndWaitForTransactionId(self, __1: SubmitAndWaitRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> SubmitAndWaitForTransactionIdResponse: ...
    def SubmitAndWaitForTransaction(self, __1: SubmitAndWaitRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> SubmitAndWaitForTransactionResponse: ...
    def SubmitAndWaitForTransactionTree(self, __1: SubmitAndWaitRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> SubmitAndWaitForTransactionTreeResponse: ...

# noinspection PyPep8Naming,DuplicatedCode
class _CommandServiceAsyncStub(CommandServiceStub):
    def SubmitAndWait(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, Empty]: ...  # type: ignore
    def SubmitAndWaitForTransactionId(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, SubmitAndWaitForTransactionIdResponse]: ...  # type: ignore
    def SubmitAndWaitForTransaction(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, SubmitAndWaitForTransactionResponse]: ...  # type: ignore
    def SubmitAndWaitForTransactionTree(self, __1: SubmitAndWaitRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, SubmitAndWaitForTransactionTreeResponse]: ...  # type: ignore
